@@ -40,10 +40,7 @@
 #include "eog-job-manager.h"
 #include "eog-image-cache.h"
 
-GNOME_CLASS_BOILERPLATE (EogFullScreen,
-			 eog_full_screen,
-			 GtkWindow,
-			 GTK_TYPE_WINDOW);
+G_DEFINE_TYPE (EogFullScreen, eog_full_screen, GTK_TYPE_WINDOW)
 
 #define POINTER_HIDE_DELAY  2   /* hide the pointer after 2 seconds */
 static GQuark JOB_ID_QUARK = 0; 
@@ -153,7 +150,10 @@ eog_full_screen_motion (GtkWidget *widget, GdkEventMotion *event)
 	/* we call so to get also the next motion event */
 	gdk_window_get_pointer (GTK_WIDGET (widget)->window, &x, &y, &mods);
 
-	result = GNOME_CALL_PARENT_WITH_DEFAULT (GTK_WIDGET_CLASS, motion_notify_event, (widget, event), FALSE);
+	if (GTK_WIDGET_CLASS (eog_full_screen_parent_class)->motion_notify_event)
+		result = GTK_WIDGET_CLASS (eog_full_screen_parent_class)->motion_notify_event (widget, event);
+	else
+		result = FALSE;
 
 	return result;
 }
@@ -318,13 +318,19 @@ check_automatic_switch (gpointer data)
 	return TRUE;
 }
 
-/* Works only for xscreensaver */
 static gboolean
 disable_screen_saver (gpointer data)
 {
 	EogFullScreenPrivate *priv;
 	gboolean success = TRUE;
-	char *argv[] = { "xscreensaver-command", "-deactivate", NULL };
+	gchar **argv;
+
+	if (g_find_program_in_path ("gnome-screensaver-command") != NULL) {
+		g_shell_parse_argv ("gnome-screensaver-command --poke", NULL, &argv, NULL);
+	}
+	else {
+		g_shell_parse_argv ("xscreensaver-command -deactivate", NULL, &argv, NULL);
+	}
 
 	priv = EOG_FULL_SCREEN (data)->priv;
 	
@@ -334,21 +340,23 @@ disable_screen_saver (gpointer data)
 		                    G_SPAWN_STDERR_TO_DEV_NULL;
 		
 		success = g_spawn_sync (NULL,  /* working directory */
-								argv,  /* command */
-								NULL,  /* environment */
-								flags, /* flags */
-								NULL,  /* child setup func */
-								NULL,  /* function data */
-								NULL,  /* standard output */
+					argv,  /* command */
+					NULL,  /* environment */
+					flags, /* flags */
+					NULL,  /* child setup func */
+					NULL,  /* function data */
+					NULL,  /* standard output */
 		                        NULL,  /* standard error */
-								NULL,  /* exit status */
-								NULL); /* GError */
+					NULL,  /* exit status */
+					NULL); /* GError */
 	}
 	
 	if (!success) {
 		priv->activity_timeout_id = 0;
-		g_print ("Disable xscreensaver failed.\n");		
+		g_print ("Disable screensaver failed.\n");		
 	}
+
+	g_strfreev (argv);
 	
 	return success;
 }
@@ -363,7 +371,7 @@ eog_full_screen_show (GtkWidget *widget)
 	fs = EOG_FULL_SCREEN (widget);
 	priv = fs->priv;
 
-	GNOME_CALL_PARENT (GTK_WIDGET_CLASS, show, (widget));
+	GTK_WIDGET_CLASS (eog_full_screen_parent_class)->show (widget);
 
 	fs->priv->have_grab = !gdk_keyboard_grab (widget->window,
 			                          TRUE, GDK_CURRENT_TIME);
@@ -383,12 +391,12 @@ eog_full_screen_show (GtkWidget *widget)
 		priv->switch_timeout_id = g_timeout_add (1000 /* every second */,
 							 check_automatic_switch,
 							 fs);
-
-		/* disable screen saver */
-		priv->activity_timeout_id = g_timeout_add (6000 /* every minute */,
-							   disable_screen_saver,
-							   fs);
 	}
+
+	/* disable screen saver */
+	priv->activity_timeout_id = g_timeout_add (6000 /* every minute */,
+						   disable_screen_saver,
+						   fs);
 }
 
 /* Hide handler for the full screen view */
@@ -414,7 +422,7 @@ eog_full_screen_hide (GtkWidget *widget)
 		fs->priv->activity_timeout_id = 0;
 	}
 
-	GNOME_CALL_PARENT (GTK_WIDGET_CLASS, hide, (widget));
+	GTK_WIDGET_CLASS (eog_full_screen_parent_class)->hide (widget);
 }
 
 
@@ -530,11 +538,19 @@ eog_full_screen_key_press (GtkWidget *widget, GdkEventKey *event)
 	}
 
 	if (do_hide) {
+		cancel_load_by_iter (priv->list, priv->preload);
+		if (priv->preload != NULL) {
+			g_free (priv->preload);
+			priv->preload = NULL;
+		}
 		gtk_widget_hide (widget);
 	}
 
 	if (!handled) {
-		handled = GNOME_CALL_PARENT_WITH_DEFAULT (GTK_WIDGET_CLASS, key_press_event, (widget, event), FALSE);
+		if (GTK_WIDGET_CLASS (eog_full_screen_parent_class)->key_press_event)
+			handled = GTK_WIDGET_CLASS (eog_full_screen_parent_class)->key_press_event (widget, event);
+		else
+			handled = FALSE;
 	}
 
 	return handled;
@@ -577,7 +593,7 @@ eog_full_screen_destroy (GtkObject *object)
 		priv->cache = NULL;
 	}
 
-	GNOME_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
+	GTK_OBJECT_CLASS (eog_full_screen_parent_class)->destroy (object);
 }	
 
 /* Finalize handler for the full screen view */
@@ -593,7 +609,7 @@ eog_full_screen_finalize (GObject *object)
 
 	g_free (fs->priv);
 
-	GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
+	G_OBJECT_CLASS (eog_full_screen_parent_class)->finalize (object);
 }
 
 /* Class initialization function for the full screen mode */
@@ -608,7 +624,6 @@ eog_full_screen_class_init (EogFullScreenClass *class)
 	gobject_class = (GObjectClass *) class;
 	widget_class = (GtkWidgetClass *) class;
 
-	parent_class = gtk_type_class (GTK_TYPE_WINDOW);
 
 	object_class->destroy = eog_full_screen_destroy;
 	gobject_class->finalize = eog_full_screen_finalize;
@@ -621,7 +636,7 @@ eog_full_screen_class_init (EogFullScreenClass *class)
 
 /* Object initialization function for the full screen view */
 static void
-eog_full_screen_instance_init (EogFullScreen *fs)
+eog_full_screen_init (EogFullScreen *fs)
 {
 	EogFullScreenPrivate *priv;
 	priv = g_new0 (EogFullScreenPrivate, 1);
@@ -645,13 +660,10 @@ eog_full_screen_instance_init (EogFullScreen *fs)
 static void
 eog_full_screen_set_geometry (EogFullScreen *fs, GtkWindow *parent)
 {
-	EogFullScreenPrivate *priv;
 	GdkScreen *screen;
 	GdkRectangle geometry;
 	int monitor;
 	
-	priv = fs->priv;
-
 	screen = gtk_window_get_screen (GTK_WINDOW (fs));
 	monitor = gdk_screen_get_monitor_at_window (screen,
 			GTK_WIDGET (parent)->window);
@@ -685,6 +697,12 @@ job_image_load_finished  (EogJob *job, gpointer data, GError *error)
 	EogImage *current_img;
 	EogImage *preload_img;
 
+	if (eog_job_get_status (job) == EOG_JOB_STATUS_CANCELED) {
+		g_object_set_qdata (G_OBJECT (image), JOB_ID_QUARK, NULL);
+		eog_image_data_unref (image);
+		return;
+	}
+
 	if (eog_job_get_success (job))
 		eog_image_cache_add (priv->cache, image);
 
@@ -821,7 +839,7 @@ eog_full_screen_new (GtkWindow *parent,
 	style->bg    [GTK_STATE_NORMAL] = style->black;
 	style->bg_gc [GTK_STATE_NORMAL] = style->black_gc;
 	if (style->bg_pixmap [GTK_STATE_NORMAL])
-		gdk_pixmap_unref (style->bg_pixmap [GTK_STATE_NORMAL]);
+		g_object_unref (style->bg_pixmap [GTK_STATE_NORMAL]);
 	style->bg_pixmap [GTK_STATE_NORMAL] = NULL;
 
 	gtk_widget_set_style (widget, style);
