@@ -1174,22 +1174,23 @@ eog_scroll_view_button_press_event (GtkWidget *widget, GdkEventButton *event, gp
 		return FALSE;
 
 	switch (event->button) {
-	case 1:
-		if (is_image_movable (view)) {
-			priv->cursor_type = CURSOR_HAND_CLOSED;
-			cursor_set (GTK_WIDGET (priv->display), CURSOR_HAND_CLOSED);
+		case 1:
+		case 2:
+			if (is_image_movable (view)) {
+				priv->cursor_type = CURSOR_HAND_CLOSED;
+				cursor_set (GTK_WIDGET (priv->display), CURSOR_HAND_CLOSED);
 
-			priv->dragging = TRUE;
-			priv->drag_anchor_x = event->x;
-			priv->drag_anchor_y = event->y;
-			
-			priv->drag_ofs_x = priv->xofs;
-			priv->drag_ofs_y = priv->yofs;
+				priv->dragging = TRUE;
+				priv->drag_anchor_x = event->x;
+				priv->drag_anchor_y = event->y;
 
-			return TRUE;
-		}
-	default:
-		break;
+				priv->drag_ofs_x = priv->xofs;
+				priv->drag_ofs_y = priv->yofs;
+
+				return TRUE;
+			}
+		default:
+			break;
 	}
 
 	return FALSE;
@@ -1218,19 +1219,26 @@ eog_scroll_view_button_release_event (GtkWidget *widget, GdkEventButton *event,
 	view = EOG_SCROLL_VIEW (data);
 	priv = view->priv;
 
-	if (!priv->dragging || event->button != 1)
+	if (!priv->dragging)
 		return FALSE;
 
-	drag_to (view, event->x, event->y);
-	priv->dragging = FALSE;
+	switch (event->button) {
+		case 1:
+		case 2:
+			drag_to (view, event->x, event->y);
+			priv->dragging = FALSE;
 
-	if (is_image_movable (view)) {
-		priv->cursor_type = CURSOR_HAND_OPEN;
-		cursor_set (GTK_WIDGET (priv->display), CURSOR_HAND_OPEN);
-	}
-	else {
-		priv->cursor_type = CURSOR_DEFAULT;
-		cursor_set (GTK_WIDGET (priv->display), CURSOR_DEFAULT);
+			if (is_image_movable (view)) {
+				priv->cursor_type = CURSOR_HAND_OPEN;
+				cursor_set (GTK_WIDGET (priv->display), CURSOR_HAND_OPEN);
+			}
+			else {
+				priv->cursor_type = CURSOR_DEFAULT;
+				cursor_set (GTK_WIDGET (priv->display), CURSOR_DEFAULT);
+			}
+
+		default:
+			break;
 	}
 
 	return TRUE;
